@@ -35,40 +35,51 @@
 
 void example1()
 {
+    //!code-start: declare
     using mtv_type = mdds::multi_type_vector<mdds::mtv::element_block_func>;
     using collection_type = mdds::mtv::collection<mtv_type>;
 
     std::vector<mtv_type> columns(5);
+    //!code-end: declare
 
+    //!code-start: header-row
     // Populate the header row.
-    auto headers = { "ID", "Make", "Model", "Year", "Color" };
+    std::array headers = { "ID", "Make", "Model", "Year", "Color" };
     size_t i = 0;
     std::for_each(headers.begin(), headers.end(), [&](const char* v) { columns[i++].push_back<std::string>(v); });
+    //!code-end: header-row
 
+    //!code-start: column-1
     // Fill column 1.
-    auto c1_values = { 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20 };
+    std::array c1_values = { 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20 };
     std::for_each(c1_values.begin(), c1_values.end(), [&columns](int v) { columns[0].push_back(v); });
+    //!code-end: column-1
 
+    //!code-start: column-2
     // Fill column 2.
-    auto c2_values =
+    std::array c2_values =
     {
         "Nissan", "Mercedes-Benz", "Nissan", "Suzuki", "Saab", "Subaru", "GMC", "Mercedes-Benz", "Toyota", "Nissan",
         "Mazda", "Dodge", "Ford", "Bentley", "GMC", "Audi", "GMC", "Mercury", "Pontiac", "BMW",
     };
 
     std::for_each(c2_values.begin(), c2_values.end(), [&columns](const char* v) { columns[1].push_back<std::string>(v); });
+    //!code-end: column-2
 
+    //!code-start: column-3
     // Fill column 3.
-    auto c3_values =
+    std::array c3_values =
     {
         "Frontier", "W201", "Frontier", "Equator", "9-5", "Tribeca", "Yukon XL 2500", "E-Class", "Camry Hybrid", "Frontier",
         "MX-5", "Ram Van 1500", "Edge", "Azure", "Sonoma Club Coupe", "S4", "3500 Club Coupe", "Villager", "Sunbird", "3 Series",
     };
 
     std::for_each(c3_values.begin(), c3_values.end(), [&columns](const char* v) { columns[2].push_back<std::string>(v); });
+    //!code-end: column-3
 
+    //!code-start: column-4
     // Fill column 4.  Replace -1 with "unknown".
-    std::initializer_list<int32_t> c4_values =
+    std::array c4_values =
     {
         1998, 1986, 2009, -1, -1, 2008, 2009, 2008, 2010, 2001,
         2008, 2000, -1, 2009, 1998, 2013, 1994, 2000, 1990, 1993,
@@ -82,19 +93,25 @@ void example1()
         else
             columns[3].push_back(v);
     }
+    //!code-end: column-4
 
+    //!code-start: column-5
     // Fill column 5
-    auto c5_values =
+    std::array c5_values =
     {
         "Turquoise", "Fuscia", "Teal", "Fuscia", "Green", "Khaki", "Pink", "Goldenrod", "Turquoise", "Yellow",
         "Orange", "Goldenrod", "Fuscia", "Goldenrod", "Mauv", "Crimson", "Turquoise", "Teal", "Indigo", "LKhaki",
     };
 
     std::for_each(c5_values.begin(), c5_values.end(), [&columns](const char* v) { columns[4].push_back<std::string>(v); });
+    //!code-end: column-5
 
+    //!code-start: wrap
     // Wrap the columns with the 'collection'...
     collection_type rows(columns.begin(), columns.end());
+    //!code-end: wrap
 
+    //!code-start: traverse-row
     // Traverse the tabular data in row-wise direction.
     for (const auto& cell : rows)
     {
@@ -119,12 +136,16 @@ void example1()
             // We are in the last column. Insert a line break.
             std::cout << std::endl;
     }
+    //!code-end: traverse-row
 
+    //!code-start: limit-range
     rows.set_collection_range(1, 2); // only columns 2 and 3.
     rows.set_element_range(1, 10);   // only rows 2 through 11.
+    //!code-end: limit-range
 
     std::cout << "--" << std::endl;
 
+    //!code-start: traverse-row-range
     for (const auto& cell : rows)
     {
         if (cell.index > 1)
@@ -148,13 +169,18 @@ void example1()
             // We are in the last column. Insert a line break.
             std::cout << std::endl;
     }
+    //!code-end: traverse-row-range
 }
 
-int main()
+int main() try
 {
     example1();
 
     return EXIT_SUCCESS;
 }
+catch (...)
+{
+    return EXIT_FAILURE;
+}
 
 /* vim:set shiftwidth=4 softtabstop=4 expandtab: */
