@@ -26,6 +26,7 @@
  *
  ************************************************************************/
 
+//!code-start
 #include <mdds/multi_type_vector.hpp>
 #include <mdds/multi_type_vector/trait.hpp>
 #include <iostream>
@@ -38,11 +39,13 @@ class event_hdl
 public:
     void element_block_acquired(mdds::mtv::base_element_block* block)
     {
+        (void)block;
         cout << "  * element block acquired" << endl;
     }
 
     void element_block_released(mdds::mtv::base_element_block* block)
     {
+        (void)block;
         cout << "  * element block released" << endl;
     }
 };
@@ -56,7 +59,7 @@ struct trait
 
 using mtv_type = mdds::multi_type_vector<mdds::mtv::element_block_func, trait>;
 
-int main()
+int main() try
 {
     mtv_type db;  // starts with an empty container.
 
@@ -76,5 +79,10 @@ int main()
 
     return EXIT_SUCCESS;
 }
+catch (...)
+{
+    return EXIT_FAILURE;
+}
+//!code-end
 
 /* vim:set shiftwidth=4 softtabstop=4 expandtab: */
