@@ -35,6 +35,10 @@
 #include "./block_util.hpp"
 #include "./iterator.hpp"
 
+#ifdef MDDS_MULTI_TYPE_VECTOR_DEBUG
+#include <iostream>
+#endif
+
 namespace mdds { namespace mtv { namespace soa {
 
 /**
@@ -113,6 +117,7 @@ private:
 
         blocks_type();
         blocks_type(const blocks_type& other);
+        blocks_type(blocks_type&& other);
 
         void pop_back()
         {
@@ -390,10 +395,17 @@ public:
     /**
      * Copy constructor.
      *
-     * @param other other column instance to copy values from.
+     * @param other the other instance to copy values from.
      */
     multi_type_vector(const multi_type_vector& other);
 
+    /**
+     * Move constructor.
+     *
+     * @param other the other instance to move values from.
+     */
+    multi_type_vector(multi_type_vector&& other);
+
     /**
      * Destructor.  It deletes all allocated element blocks.
      */
@@ -1100,6 +1112,7 @@ public:
     bool operator!= (const multi_type_vector& other) const;
 
     multi_type_vector& operator= (const multi_type_vector& other);
+    multi_type_vector& operator= (multi_type_vector&& other);
 
     /**
      * Return the numerical identifier that represents passed element.
@@ -1122,6 +1135,9 @@ private:
 
     void delete_element_blocks(size_type start, size_type end);
 
+    template<typename T>
+    void get_impl(size_type pos, T& value) const;
+
     template<typename T>
     bool set_cells_precheck(
         size_type row, const T& it_begin, const T& it_end, size_type& end_pos);
@@ -1430,6 +1446,10 @@ private:
     event_func m_hdl_event;
     blocks_type m_block_store;
     size_type m_cur_size;
+
+#ifdef MDDS_MULTI_TYPE_VECTOR_DEBUG
+    mutable int m_trace_call_depth = 0;
+#endif
 };
 
 }}}
