@@ -1,6 +1,6 @@
 /*************************************************************************
  *
- * Copyright (c) 2010-2014 Kohei Yoshida
+ * Copyright (c) 2010-2015 Kohei Yoshida
  *
  * Permission is hereby granted, free of charge, to any person
  * obtaining a copy of this software and associated documentation
@@ -25,20 +25,17 @@
  *
  ************************************************************************/
 
-#ifndef __MDDS_SEGMENTTREE_HPP__
-#define __MDDS_SEGMENTTREE_HPP__
+#ifndef INCLUDED_MDDS_SEGMENTTREE_HPP
+#define INCLUDED_MDDS_SEGMENTTREE_HPP
 
 #include "mdds/node.hpp"
-#include "mdds/hash_container/map.hpp"
 #include "mdds/global.hpp"
 
 #include <vector>
-#include <list>
 #include <iostream>
 #include <map>
-
-#include <boost/shared_ptr.hpp>
-#include <boost/ptr_container/ptr_map.hpp>
+#include <unordered_map>
+#include <memory>
 
 #ifdef MDDS_UNIT_TEST
 #include <sstream>
@@ -46,110 +43,23 @@
 
 namespace mdds {
 
-template<typename _Key, typename _Data>
-class rectangle_set;
-
-namespace __st {
-
-template<typename T, typename _Inserter>
-void descend_tree_for_search(
-    typename T::key_type point, const __st::node_base* pnode, _Inserter& result)
-{
-    typedef typename T::node leaf_node;
-    typedef typename T::nonleaf_node nonleaf_node;
-
-    typedef typename T::key_type key_type;
-    typedef typename T::nonleaf_value_type nonleaf_value_type;
-    typedef typename T::leaf_value_type leaf_value_type;
-    typedef _Inserter inserter_type;
-
-    if (!pnode)
-        // This should never happen, but just in case.
-        return;
-
-    if (pnode->is_leaf)
-    {
-        result(static_cast<const leaf_node*>(pnode)->value_leaf.data_chain);
-        return;
-    }
-
-    const nonleaf_node* pnonleaf = static_cast<const nonleaf_node*>(pnode);
-    const nonleaf_value_type& v = pnonleaf->value_nonleaf;
-    if (point < v.low || v.high <= point)
-        // Query point is out-of-range.
-        return;
-
-    result(v.data_chain);
-
-    // Check the left child node first, then the right one.
-    __st::node_base* pchild = pnonleaf->left;
-    if (!pchild)
-        return;
-
-    assert(pnonleaf->right ? pchild->is_leaf == pnonleaf->right->is_leaf : true);
-
-    if (pchild->is_leaf)
-    {
-        // The child node are leaf nodes.
-        const leaf_value_type& vleft = static_cast<const leaf_node*>(pchild)->value_leaf;
-        if (point < vleft.key)
-        {
-            // Out-of-range.  Nothing more to do.
-            return;
-        }
-
-        if (pnonleaf->right)
-        {
-            assert(pnonleaf->right->is_leaf);
-            const leaf_value_type& vright = static_cast<const leaf_node*>(pnonleaf->right)->value_leaf;
-            if (vright.key <= point)
-                // Follow the right node.
-                pchild = pnonleaf->right;
-        }
-    }
-    else
-    {
-        // This child nodes are non-leaf nodes.
-
-        const nonleaf_value_type& vleft =
-            static_cast<const nonleaf_node*>(pchild)->value_nonleaf;
-
-        if (point < vleft.low)
-        {
-            // Out-of-range.  Nothing more to do.
-            return;
-        }
-        if (vleft.high <= point && pnonleaf->right)
-            // Follow the right child.
-            pchild = pnonleaf->right;
-
-        assert(static_cast<const nonleaf_node*>(pchild)->value_nonleaf.low <= point &&
-               point < static_cast<const nonleaf_node*>(pchild)->value_nonleaf.high);
-    }
-
-    descend_tree_for_search<T,_Inserter>(point, pchild, result);
-}
-
-} // namespace __st
-
-template<typename _Key, typename _Data>
+template<typename _Key, typename _Value>
 class segment_tree
 {
-    friend class rectangle_set<_Key, _Data>;
 public:
     typedef _Key        key_type;
-    typedef _Data       data_type;
+    typedef _Value      value_type;
     typedef size_t      size_type;
-    typedef ::std::vector<data_type> search_result_type;
+    typedef std::vector<value_type> search_results_type;
 
 #ifdef MDDS_UNIT_TEST
     struct segment_data
     {
         key_type    begin_key;
         key_type    end_key;
-        data_type   pdata;
+        value_type   pdata;
 
-        segment_data(key_type _beg, key_type _end, data_type p) :
+        segment_data(key_type _beg, key_type _end, value_type p) :
             begin_key(_beg), end_key(_end), pdata(p) {}
 
         bool operator==(const segment_data& r) const
@@ -163,9 +73,9 @@ public:
         }
     };
 
-    struct segment_map_printer : public ::std::unary_function< ::std::pair<data_type, ::std::pair<key_type, key_type> >, void>
+    struct segment_map_printer
     {
-        void operator() (const ::std::pair<data_type, ::std::pair<key_type, key_type> >& r) const
+        void operator() (const ::std::pair<value_type, ::std::pair<key_type, key_type> >& r) const
         {
             using namespace std;
             cout << r.second.first << "-" << r.second.second << ": " << r.first->name << endl;
@@ -174,9 +84,9 @@ public:
 #endif
 
 public:
-    typedef ::std::vector<data_type> data_chain_type;
-    typedef _mdds_unordered_map_type<data_type, ::std::pair<key_type, key_type> > segment_map_type;
-    typedef ::std::map<data_type, ::std::pair<key_type, key_type> >               sorted_segment_map_type;
+    typedef ::std::vector<value_type> data_chain_type;
+    typedef std::unordered_map<value_type, ::std::pair<key_type, key_type> > segment_map_type;
+    typedef ::std::map<value_type, ::std::pair<key_type, key_type> >               sorted_segment_map_type;
 
     struct nonleaf_value_type
     {
@@ -297,12 +207,12 @@ public:
     {
         void operator() (node& _self)
         {
-            _self.value_leaf.data_chain = NULL;
+            _self.value_leaf.data_chain = nullptr;
         }
 
         void operator() (__st::nonleaf_node<segment_tree>& _self)
         {
-            _self.value_nonleaf.data_chain = NULL;
+            _self.value_nonleaf.data_chain = nullptr;
         }
     };
 
@@ -320,7 +230,7 @@ public:
     };
 
 #ifdef MDDS_UNIT_TEST
-    struct node_printer : public ::std::unary_function<const __st::node_base*, void>
+    struct node_printer
     {
         void operator() (const __st::node_base* p) const
         {
@@ -338,17 +248,17 @@ private:
      * This base class takes care of collecting data chain pointers during
      * tree descend for search.
      */
-    class search_result_base
+    class search_results_base
     {
     public:
-        typedef ::std::vector<data_chain_type*>         res_chains_type;
-        typedef ::boost::shared_ptr<res_chains_type>    res_chains_ptr;
+        typedef std::vector<data_chain_type*>       res_chains_type;
+        typedef std::shared_ptr<res_chains_type>    res_chains_ptr;
     public:
 
-        search_result_base() :
-            mp_res_chains(static_cast<res_chains_type*>(NULL)) {}
+        search_results_base() :
+            mp_res_chains(static_cast<res_chains_type*>(nullptr)) {}
 
-        search_result_base(const search_result_base& r) :
+        search_results_base(const search_results_base& r) :
             mp_res_chains(r.mp_res_chains) {}
 
         size_t size() const
@@ -383,8 +293,8 @@ private:
     class iterator_base
     {
     protected:
-        typedef typename search_result_base::res_chains_type res_chains_type;
-        typedef typename search_result_base::res_chains_ptr res_chains_ptr;
+        typedef typename search_results_base::res_chains_type res_chains_type;
+        typedef typename search_results_base::res_chains_ptr res_chains_ptr;
 
         iterator_base(const res_chains_ptr& p) :
             mp_res_chains(p), m_end_pos(true) {}
@@ -397,7 +307,7 @@ private:
         typedef typename data_chain_type::difference_type   difference_type;
 
         iterator_base() :
-            mp_res_chains(static_cast<res_chains_type*>(NULL)), m_end_pos(true) {}
+            mp_res_chains(static_cast<res_chains_type*>(nullptr)), m_end_pos(true) {}
 
         iterator_base(const iterator_base& r) :
             mp_res_chains(r.mp_res_chains),
@@ -435,7 +345,7 @@ private:
                 if (cur_chain == mp_res_chains->end())
                 {
                     m_end_pos = true;
-                    return NULL;
+                    return nullptr;
                 }
                 m_cur_chain = cur_chain;
                 m_cur_pos_in_chain = (*m_cur_chain)->begin();
@@ -449,7 +359,7 @@ private:
         typename data_chain_type::value_type* operator-- ()
         {
             if (!mp_res_chains)
-                return NULL;
+                return nullptr;
 
             if (m_end_pos)
             {
@@ -462,7 +372,7 @@ private:
                 if (m_cur_chain == mp_res_chains->begin())
                 {
                     // Already at the first data chain.  Don't move the iterator position.
-                    return NULL;
+                    return nullptr;
                 }
                 --m_cur_chain;
                 m_cur_pos_in_chain = (*m_cur_chain)->end();
@@ -538,40 +448,40 @@ private:
 
 public:
 
-    class search_result : public search_result_base
+    class search_results : public search_results_base
     {
-        typedef typename search_result_base::res_chains_type res_chains_type;
-        typedef typename search_result_base::res_chains_ptr res_chains_ptr;
+        typedef typename search_results_base::res_chains_type res_chains_type;
+        typedef typename search_results_base::res_chains_ptr res_chains_ptr;
     public:
 
         class iterator : public iterator_base
         {
-            friend class segment_tree<_Key,_Data>::search_result;
+            friend class segment_tree<_Key,_Value>::search_results;
         private:
             iterator(const res_chains_ptr& p) : iterator_base(p) {}
         public:
             iterator() : iterator_base() {}
         };
 
-        typename search_result::iterator begin()
+        typename search_results::iterator begin()
         {
-            typename search_result::iterator itr(search_result_base::get_res_chains());
+            typename search_results::iterator itr(search_results_base::get_res_chains());
             itr.move_to_front();
             return itr;
         }
 
-        typename search_result::iterator end()
+        typename search_results::iterator end()
         {
-            typename search_result::iterator itr(search_result_base::get_res_chains());
+            typename search_results::iterator itr(search_results_base::get_res_chains());
             itr.move_to_end();
             return itr;
         }
     };
 
-    class search_result_vector_inserter : public ::std::unary_function<data_chain_type*, void>
+    class search_result_vector_inserter
     {
     public:
-        search_result_vector_inserter(search_result_type& result) : m_result(result) {}
+        search_result_vector_inserter(search_results_type& result) : m_result(result) {}
         void operator() (data_chain_type* node_data)
         {
             if (!node_data)
@@ -582,13 +492,13 @@ public:
                 m_result.push_back(*itr);
         }
     private:
-        search_result_type& m_result;
+        search_results_type& m_result;
     };
 
-    class search_result_inserter : public ::std::unary_function<data_chain_type*, void>
+    class search_result_inserter
     {
     public:
-        search_result_inserter(search_result_base& result) : m_result(result) {}
+        search_result_inserter(search_results_base& result) : m_result(result) {}
         void operator() (data_chain_type* node_data)
         {
             if (!node_data)
@@ -597,7 +507,7 @@ public:
             m_result.push_back_chain(node_data);
         }
     private:
-        search_result_base& m_result;
+        search_results_base& m_result;
     };
 
     segment_tree();
@@ -634,7 +544,7 @@ public:
      *               Note that <i>the caller must manage the life cycle of the
      *               data instance</i>.
      */
-    bool insert(key_type begin_key, key_type end_key, data_type pdata);
+    bool insert(key_type begin_key, key_type end_key, value_type pdata);
 
     /**
      * Search the tree and collect all segments that include a specified
@@ -652,7 +562,7 @@ public:
      * @return true if the search is performed successfully, false if the
      *         search has ended prematurely due to error conditions.
      */
-    bool search(key_type point, search_result_type& result) const;
+    bool search(key_type point, search_results_type& result) const;
 
     /**
      * Search the tree and collect all segments that include a specified
@@ -663,14 +573,16 @@ public:
      * @return object containing the result of the search, which can be
      *         accessed via iterator.
      */
-    search_result search(key_type point) const;
+    search_results search(key_type point) const;
 
     /**
-     * Remove a segment by the data pointer.  This will <i>not</i> invalidate
-     * the tree; however, if you have removed lots of segments, you might want
-     * to re-build the tree to shrink its size.
+     * Remove a segment that matches by the value.  This will <i>not</i>
+     * invalidate the tree; however, if you have removed lots of segments, you
+     * might want to re-build the tree to shrink its size.
+     *
+     * @param value value to remove a segment by.
      */
-    void remove(data_type pdata);
+    void remove(value_type value);
 
     /**
      * Remove all segments data.
@@ -712,7 +624,7 @@ public:
      * Verify the validity of the segment data array.
      *
      * @param checks null-terminated array of expected values.  The last item
-     *               must have a NULL pdata value to terminate the array.
+     *               must have a nullptr pdata value to terminate the array.
      */
     bool verify_segment_data(const segment_map_type& checks) const;
 #endif
@@ -721,10 +633,10 @@ private:
     /**
      * To be called from rectangle_set.
      */
-    void search(key_type point, search_result_base& result) const;
+    void search(key_type point, search_results_base& result) const;
 
-    typedef ::std::vector<__st::node_base*> node_list_type;
-    typedef ::boost::ptr_map<data_type, node_list_type> data_node_map_type;
+    typedef std::vector<__st::node_base*> node_list_type;
+    typedef std::map<value_type, std::unique_ptr<node_list_type>> data_node_map_type;
 
     static void create_leaf_node_instances(const ::std::vector<key_type>& keys, node_ptr& left, node_ptr& right);
 
@@ -734,7 +646,7 @@ private:
      * record their positions as a list of node pointers.
      */
     void descend_tree_and_mark(
-        __st::node_base* pnode, data_type pdata, key_type begin_key, key_type end_key, node_list_type* plist);
+        __st::node_base* pnode, value_type pdata, key_type begin_key, key_type end_key, node_list_type* plist);
 
     void build_leaf_nodes();
 
@@ -742,13 +654,13 @@ private:
      * Go through the list of nodes, and remove the specified data pointer
      * value from the nodes.
      */
-    void remove_data_from_nodes(node_list_type* plist, const data_type pdata);
-    void remove_data_from_chain(data_chain_type& chain, const data_type pdata);
+    void remove_data_from_nodes(node_list_type* plist, const value_type pdata);
+    void remove_data_from_chain(data_chain_type& chain, const value_type pdata);
 
     void clear_all_nodes();
 
 #ifdef MDDS_UNIT_TEST
-    static bool has_data_pointer(const node_list_type& node_list, const data_type pdata);
+    static bool has_data_pointer(const node_list_type& node_list, const value_type pdata);
     static void print_leaf_value(const leaf_value_type& v);
 #endif
 
